# -*- coding: utf-8 -*-
"""PythonShell Core"""
import re
import sys

"""Python 2 support"""
if sys.version_info < (3,):
    input = raw_input

from pythonshell import api, info
from pythonshell.exceptions import ShellException

class Shell(object):
    """PythonShell core class. Extend this to add custom commands"""

    def __init__(self, name=info.name, sign='$',
            version=info.version, *args, **kwargs):
        """Sets the name, sign, version and any other variables"""
        self.name = name
        self.sign = sign
        self.version = version
        self.cmd_variables = {}

        for arg, value in kwargs:
            setattr(self, arg, value)

    def start(self):
        """Starts the shell"""
        while True:
            try:
                variable = False
                cmd = input('%s%s ' % (self.name, self.sign))
                # Check if cmd is a variable
                if cmd in self.cmd_variables:
                    print(self.cmd_variables[cmd])
                    continue
                # Check if a variable was defined and if has a valid name
                if re.match('^\s{,}[a-zA-Z]{1,}\s{,}[=]{1}', cmd):
                    variable = cmd.split('=')[0].strip()
                    cmd = cmd[cmd.index('=')+1:].strip()
                    
                    if self.check_shell_method(variable):
                        raise ShellException('%s is a reserved word' %
                                variable)
                    elif not self.check_shell_method(cmd):
                        self.cmd_variables[variable] = cmd
                        continue
                    
                cmd = cmd.strip().split(' ')
                arguments = cmd[1:]
                cmd = cmd[0].strip()
                pos_args = []
                named_args = {}
                for argument in arguments:
                    # Check if it's a named argument
                    if re.match('^\w{,}\s{,}[=]{1}\s{,}\w{,}$', argument):
                        argument_split = argument.split('=')
                        named_args[argument_split[0].strip()] =\
                                argument_split[1].strip()
                    else:
                        pos_args.append(argument.strip())
                

                if not self.check_shell_method(cmd, hidden=True):
                    raise ShellException('%s isn\'t a valid command'
                            'or variable' % cmd)
                method = getattr(self, cmd)
                result = method(*pos_args, **named_args)

                if variable:
                    self.cmd_variables[variable] = result
            except (KeyboardInterrupt, EOFError):
                break
            except ShellException as Ex:
                print('shell: %s' % Ex)
            except Exception as Ex:
                if not cmd:
                    cmd = 'Unknown Error'
                print('%s: %s' % (cmd, Ex))

    def check_shell_method(self, method, hidden=False):
        """Checks the method is a valid shell command"""
        def _check(field):
            if not method or not field:
                return False
            else:
                return hasattr(method, field) and getattr(method, field)

        ismethod = hasattr(self, method) and callable(getattr(self, method))
        if not ismethod:
            return False
        method = getattr(self, method)
        return _check('_shell') and (hidden or not _check('_hidden'))
    
    def get_shell_methods(self):
        """Get all available commands, except hidden ones"""
        methods = []
        for attribute in sorted(dir(self)):
            if self.check_shell_method(attribute):
                methods.append(getattr(self, attribute))
        return methods

    @api.shell
    def help(self, *args, **kwargs):
        """Prints this help message"""
        print ('Available commands\n')
        
        for method in self.get_shell_methods():
            print('%s: %s' % (method.__name__, method.__doc__))

        print ('')
        return True

    @api.shell
    def about(self, *args, **kwargs):
        """About"""
        return self.aboutshell()

    @api.hidden
    def aboutshell(self, *args, **kwargs):
        """Prints PythonShell information"""
        print (info.brand)
        print ('\nVersion %s %s' % (info.version, info.codename))
        print (info.description)

        print ('\nBrought to you by %s <%s>' % (info.author, info.author_email))
        print ('See more at %s' % info.url)

        print ('\n%s is under %s' % (info.name, info.license))
        print ('Copyright %s %s' % (info.build_year, info.author))

        print ('\nWith love from Lisbon\n')

        return True

    @api.shell
    def unset(self, *args, **kwargs):
        """Delete shell variables"""
        if len(args) == 0:
            return True
        for variable in args:
            if variable in self.cmd_variables:
                del self.cmd_variables[variable]
        return True

    @api.shell
    def variables(self, *args, **kwargs):
        """Lists variables"""
        for variable, value in self.cmd_variables.items():
            print('%s: %s' % (variable, value))
        return True

    @api.shell
    def exit(self, *args, **kwargs):
        """Exists shell"""
        raise KeyboardInterrupt
